/*------------------------------------------------------------------------------
--  This file is a part of the libuc, microcontroler library
--  Copyright (C) 2012, Alexis Jeandet
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                 Author : Alexis Jeandet
--                   Mail : alexis.jeandet@gmail.com
-------------------------------------------------------------------------------*/
#include "fat32.h"
#include "libucstrings.h"

char fat32buff[512];
unsigned int lastSecAddrs=-1;
FAT32fs* lastFATFS=0;

int fat32open(FAT32fs* fs,dikpartition* part)
{
	char sector[512];
	fs->part = part;
	if(!isFat32(fs->part->TypeCode))return FATBadpart;
	if(fs->part->phy->read(fs->part->phy,sector,fs->part->LBABegin,1)!=RES_OK)return FATReadErr;
	fs->BPB_BytsPerSec = sector[BPB_BytsPerSecoff] + (sector[BPB_BytsPerSecoff+1]<<8);
	fs->BPB_RsvdSecCnt = sector[BPB_RsvdSecCntoff] + (sector[BPB_RsvdSecCntoff+1]<<8);
	fs->BPB_FATSz32    = sector[BPB_FATSz32off] + (sector[BPB_FATSz32off+1]<<8) + (sector[BPB_FATSz32off+2]<<16) + (sector[BPB_FATSz32off+3]<<24);
	fs->BPB_RootClus   = sector[BPB_RootClusoff] + (sector[BPB_RootClusoff+1]<<8) + (sector[BPB_RootClusoff+2]<<16) + (sector[BPB_RootClusoff+3]<<24);
	fs->BPB_SecPerClus = sector[BPB_SecPerClusoff];
	fs->BPB_NumFATs    = sector[BPB_NumFATsoff];
	fs->fat_begin_lba  = (unsigned int)fs->part->LBABegin + (unsigned int)(unsigned short)fs->BPB_RsvdSecCnt;
	fs->cluster_begin_lba = (unsigned int)fs->fat_begin_lba + ((unsigned int)fs->BPB_NumFATs * (unsigned int)fs->BPB_FATSz32 );
	return FATnoErr;
}

int fat32mkdirent(FAT32fs* fs,ucdirent* dirent)
{
	if(fs==0) return DIRENT_BabArg;
	if(dirent==0) return DIRENT_BabArg;
	dirent->fs = (UHANDLE)fs;
	dirent->getrootfirstent = &fat32getrootfirstent;
	dirent->nextdirent = &fat32nextdirent;
	return DIRENT_noErr;
}


int fat32getrootfirstent(ucdirent* entry)
{
	entry->CurrentSec = ((FAT32fs*)entry->fs)->cluster_begin_lba;
	entry->Currententry = -1;
	return fat32nextdirent(entry);
}


int fat32nextdirent(ucdirent* entry)
{
	int n=0,res=DIRENT_noErr,k,j;
	k=(((signed char)entry->Currententry)+1)*32; /*Restart at last dir entry*/
	j=entry->CurrentSec;//-((FAT32fs*)entry->fs)->cluster_begin_lba;  /*Restart at last sector*/
	do
	{
		if((lastSecAddrs!=j) || (lastFATFS!=((FAT32fs*)entry->fs)))
		{
			if(((FAT32fs*)entry->fs)->part->phy->read(((FAT32fs*)entry->fs)->part->phy,fat32buff,j,1)!=RES_OK)
				{
					return DIRENT_ReadErr;
				}
		}
		for(k=k;k<(512);k+=32)     /*512 Bytes per sector and 32 bytes per entry*/
		{
			if(fat32buff[DIR_Nameoff+k]!=0xE5)   /*0xE5 = free entry*/
			{
				if(fat32buff[DIR_Nameoff+k]==0) return DIRENT_EndOfDir;   /*0x00 = end of dir*/
				else 
					if(fat32buff[DIR_Attroff + k]!=ATTR_LONGNAME)
					{
						for(n=0;n<11;n++)entry->DIR_Name[n] = fat32buff[n+DIR_Nameoff + k];
						entry->DIR_Attr         = fat32buff[DIR_Attroff + k];
						entry->DIR_CrtTimeTenth = fat32buff[DIR_CrtTimeTenthoff + k];
						entry->DIR_CrtTime      = fat32buff[DIR_CrtTimeoff + k] + (fat32buff[DIR_CrtTimeoff+1 + k]<<8);
						entry->DIR_CrtDate      = fat32buff[DIR_CrtDateoff + k] + (fat32buff[DIR_CrtDateoff+1 + k]<<8);
						entry->DIR_LstAccDate   = fat32buff[DIR_LstAccDateoff + k] + (fat32buff[DIR_LstAccDateoff+1 + k]<<8);
						entry->DIR_FstClusHI    = fat32buff[DIR_FstClusHIoff + k] + (fat32buff[DIR_FstClusHIoff+1 + k]<<8);
						entry->DIR_WrtTime      = fat32buff[DIR_WrtTimeoff + k] + (fat32buff[DIR_WrtTimeoff+1 + k]<<8);
						entry->DIR_FstClustLO   = fat32buff[DIR_FstClusLOoff + k] + (fat32buff[DIR_FstClusLOoff+1 + k]<<8);
						entry->DIR_FileSize     = fat32buff[DIR_FileSizeoff + k] + (fat32buff[DIR_FileSizeoff+1 + k]<<8) + (fat32buff[DIR_FileSizeoff+2 + k]<<16) + (fat32buff[DIR_FileSizeoff+3 + k]<<24);
						entry->CurrentSec = j;
						entry->Currententry = (unsigned char)(k/32);
						return DIRENT_noErr;
					}
			}
			
		}
		j=fat32nextsectorlba(((FAT32fs*)entry->fs),j,&k);
		if(j!=DIRENT_noErr)
		{
			if(j==DIRENT_LastSect) return DIRENT_EndOfDir;   /*Not clean but can avoid some bugs*/
			return j;
		}
		j=k;
		k=0;
	}while(res==DIRENT_noErr);
	return res;
}



unsigned int fat32getdirentlba(ucdirent* entry)
{
	unsigned int culsterNum = (unsigned int)((unsigned short)entry->DIR_FstClustLO) + ((unsigned int)((unsigned short)entry->DIR_FstClusHI) <<16);
	unsigned int lba=clusterlba(((FAT32fs*)entry->fs),culsterNum);
	return lba;
}

int fat32nextsectorlba(FAT32fs* fs,unsigned int lastsector_lba,unsigned int* nextsector_lba)
{

	if((lastsector_lba + 1 - fs->cluster_begin_lba & (fs->BPB_SecPerClus-1))!=0)  /*Is it the last sector of the cluster?*/
	{
		*nextsector_lba = lastsector_lba+1;
		libucprintf("Next sector is %x\n\r",*nextsector_lba);
		return DIRENT_noErr;
	}
	int clusternum=fat32sectorlbatoclusternum(fs,lastsector_lba); /*Get cluster Number from sector number*/
	int fatsec=fat32clusterinfatsect(fs,clusternum);  /*Get FAT sector number*/
	if((lastSecAddrs!=fatsec) || (lastFATFS!=fs))  /*Check if sector already buffered*/
	{
		if(fs->part->phy->read(fs->part->phy,fat32buff,fatsec,1)!=RES_OK)
		{
			return DIRENT_ReadErr;
		}
	}
	lastSecAddrs=fatsec;
	lastFATFS=fs;
	char i= fat32clusterinfatoff(clusternum);
	*nextsector_lba = fat32buff[i] + (fat32buff[i+1]<<8) + (fat32buff[i+2]<<16) + (fat32buff[i+3]<<24);
	if(*nextsector_lba>=0xFFFFFFF8)return DIRENT_LastSect;  
	if(*nextsector_lba==0xFFFFFFF7)return DIRENT_BadSect; 
	*nextsector_lba = clusterlba(fs,*nextsector_lba);
	return DIRENT_noErr;
}

























